function Pred = SlotAvgSpatialE7(modelParms, Data, expParms)
% Slot-and-averaging model of Zhang & Luck - version with spatial
% imprecision -> transpositions
% Variant in which transpositions occur at encoding, and guessing occurs
% when the probed location is not in a slot
% The probed location is encoded in a slot with probability 1-gamma, and
% given that that is the case, pencode represents the probability of each
% object being combined with the probed location in that slot

% This model is fit to all 3 cueing conditions simultaneously

cueIndex = [-1, 1, 0];    %codes in data set for neutral, valid, invalid   

k = modelParms(1);   %number of slots
colorSigma = 2*pi*modelParms(2)/360;             % standard deviation of each slot for color feature, converted from degrees into radians
s = modelParms(3);
Setsize = unique(expParms.setsize)';
slots4cued = zeros(1,max(Setsize));
slots4cued(Setsize(2:end)) = modelParms(4:7); %proportion of slots assigned to cued item for set sizes 2, 4, 6, 8
slots4cued = k*slots4cued;                    %number of slots assigned to cued item... 

pretrieve = zeros(3,max(Setsize));
Setsize = unique(expParms.setsize)';
cSigmaList = ones(3,8)*colorSigma;

%determine for each set size how many slots, on average, are assigned to an
%item, given a value of k, and compute SD (sigma) from that
for sz = Setsize
    % neutral condition
    slotproportion(1) = k/sz;  %proportion of slots available for each item (e.g., if k = 5, and sz = 2, slotproportion = 2.5)
    pretrieve(1, sz) = min(1, k./sz);  %probability that the target location has received a slot, so that something is retrieved 
    % valid condition
    if sz == 1, 
        slotproportion(2) = k; 
        pretrieve(2,sz) = min(1, k/sz); 
    else
        slotproportion(2) = slots4cued(sz); 
        pretrieve(2, sz) = min(1, slots4cued(sz));
    end
    % invalid condition
    if sz == 1,   %irrelevant, this does not happen
        slotproportion(3) = 1; 
        pretrieve(3,sz) = 0;
    else
        slotproportion(3) = max(0, k-slots4cued(sz))/sz; 
        pretrieve(3, sz) = min(1, max(0, (k-slots4cued(sz))/sz));
    end

    for cuecond = 1:3
        if slotproportion(cuecond) > 1
            wholeslots = floor(slotproportion(cuecond));  %number of whole slots assigned to every item (in the example, every item gets at least 2 slots)
            rest = slotproportion(cuecond)-wholeslots;    %as yet unassigned part of slotproportion (e.g., 0.5), equals the proportion of items that receive one extra slot
            cSigmaList(cuecond,sz) = (1-rest)*colorSigma./sqrt(wholeslots) + rest*colorSigma/sqrt(wholeslots+1);  %a proportion (1-rest) of items receive wholeslots, a proportion rest of all items receives one additional slot
        end
    end

end

Pretrieve = zeros(size(expParms.setsize));
cKappa = zeros(size(Data.D, 1), 1);

for sz = Setsize
    for cuecond = 1:3
        Pretrieve(expParms.setsize==sz & expParms.cue==cueIndex(cuecond)) = pretrieve(cuecond, sz);
        cKappa(expParms.setsize==sz & expParms.cue==cueIndex(cuecond)) = sd2k(cSigmaList(cuecond,sz));
    end
end

Encode = exp(-s*Data.D);     % strength of tendency to encode item j in conjunction with the probed location (not normalized, so not yet the probability!)
Pchoose = VonMisesN(Data.Dcang, 0, repmat(cKappa, [1, size(Data.Dcang,2), size(Data.Dcang,3)]), 3); %for each trial and each item j in that trial, probability of choosing color i, given item j is retrieved

Pred = zeros(size(Data.D, 1), 360);
for tIndex = 1:size(Data.D, 1),
    Pencode = Encode(tIndex,1:expParms.setsize(tIndex))./sum(Encode(tIndex,1:expParms.setsize(tIndex))); %normalize Encode to obtain probability of encoding item j in conjunction with probed location
    if expParms.setsize(tIndex) == 1, Pred(tIndex,:) = squeeze(Pchoose(tIndex, 1, :)); 
    else
        Pred(tIndex,:) = repmat(Pretrieve(tIndex), 1, 360) .* squeeze(sum(bsxfun(@times, Pencode, Pchoose(tIndex,1:expParms.setsize(tIndex),:))))' + ...
            repmat(1-Pretrieve(tIndex), 1, 360).*(1/360);
    end
end
help = 1;
