% compute predictions for 7-Parameter Variable Precision Model for Color Wheel -
% simultaneous fitting of the 3 cueing conditions, assuming variable
% (across set-size) proportion of resource is assigned to cued item, 
% the remainder is distributed evenly to all items (including the cued one)

function output = VP7(modelParms,Data,expParms)

fullResourceJbar  = modelParms(1);       % mean kappa when all the resource are allocated to one item
resourceAllocatedtoCue = ones(1,5);
resourceAllocatedtoCue(2:5)  = modelParms(2:5);       % proportion of resource allocated to cued item for set sizes 2, 4, 6, 8
tau = modelParms(6);       % scale of Gamma distribution for kappas
alpha = modelParms(7);     % exponent of the power function for kappas over setsize

nsim = 5000;  % number of simulations
stream = RandStream('mt19937ar','Seed',28);
RandStream.setGlobalStream(stream);

% define some constant
NEUTRALINDEX = 1;
VALIDINDEX = 3;
INVALIDINDEX = 2; % correction KO 14. Feb 2014: -1 = neutral, 1 = valid, 0 = invalid!

szList = unique(expParms.setsize)';

Jbar(NEUTRALINDEX, :) = fullResourceJbar ./ (szList.^alpha); % in neutral condition, resource is evenly allocated to all the items
Jbar(INVALIDINDEX, :) = fullResourceJbar * (1-resourceAllocatedtoCue) ./ (szList.^alpha);
Jbar(VALIDINDEX, :) = fullResourceJbar * resourceAllocatedtoCue + Jbar(INVALIDINDEX, :);

if expParms.mode == 1
	vPDFSz = zeros([5, 3, 360]); % PDF for each set size/conditions
    for szInd = 1:length(szList)
        for condInd = 1:3
            J = gamrnd(repmat(Jbar(condInd, szInd), nsim, 1)/tau, tau);
            J = min(J,max(expParms.J_map));
            kappa = interp1(expParms.J_map,expParms.kappa_map,J);
            cumPDF = zeros(1,360);
            for sim = 1:nsim
                vPDF = VonMisesN([0:359]*2*pi/360, 0, kappa(sim));
                if sum(vPDF) == 0 || any(isnan(vPDF)), vPDF(:) = eps; vPDF(1) = 1; end %when kappa > 700, vonMises evaluates all to 0 -> set vPDF to 1 at the target location
                cumPDF = cumPDF + vPDF;
            end
            vPDFSz(szInd, condInd, :) = cumPDF./sum(cumPDF);
        end
    end
	
    likelihood = zeros(expParms.N, 1); 
	for trial = 1:expParms.N
		szInd = find(szList == expParms.setsize(trial), 1);
		condInd = Data.cueCondition(trial)+2;
		likelihood(trial) = vPDFSz(szInd, condInd, Data.Dev(trial)+1);
	end
	output = likelihood;
end

if expParms.mode == 2  %calculate predicted probability of choosing for all 360 colors
	vPDFSz = zeros([5, 3, 360]); % PDF for each set size/conditions    
    for szInd = 1:length(szList)
        for condInd = 1:3
            J = gamrnd(repmat(Jbar(condInd, szInd), nsim, 1)/tau, tau);
            J = min(J,max(expParms.J_map));
            kappa = interp1(expParms.J_map,expParms.kappa_map,J);
            cumPDF = zeros(1,360);
            for sim = 1:nsim
                vPDF = VonMisesN([0:359]*2*pi/360, 0, kappa(sim));
                if sum(vPDF) == 0 || any(isnan(vPDF)), vPDF(:) = eps; vPDF(1) = 1; end %when kappa > 700, vonMises evaluates all to 0 -> set vPDF to 1 at the target location
                cumPDF = cumPDF + vPDF;
            end
            vPDFSz(szInd, condInd, :) = cumPDF./sum(cumPDF);
        end
    end
            
    Pchoose = zeros(expParms.N, 360);
	for trial = 1:expParms.N
		targetInd = find(Data.dtarget(trial, :) == 0, 1);	
		szInd = find(szList == expParms.setsize(trial), 1);
		condInd = Data.cueCondition(trial)+2;
        Pchoose(trial,:) = circshift(squeeze(vPDFSz(szInd, condInd, :))', [0, targetInd-1]);  % shifts the vMises with peak at 1 such that the peak is at targetInd
	end
    Pred = Pchoose;
    likelihood = zeros(expParms.N,1);
    for trial = 1:expParms.N
        likelihood(trial) = Pred(trial, Data.responsegrad(trial));
    end
    output.Pred = Pred;
    output.likelihood = sum(-2*log(likelihood));
end

